const capcon = require("capture-console"); null;;

const Loggers = require('@goodware/log');

const options = {
  stage: process.env.NODE_ENV, // This is the value for the 'stage' key that appears in all
                               // log entries (aka meta key)
  service: 'Example', // The name of this program (another meta key)
  version: '1.0.0', // The program's version (another meta key)
  console: {
    colors: true, // Create visually appealing console entries with ANSI colors. The color of each
                  // logging level can be specified. true is the default.
    data: true, // Send all logged data (including stack traces) to the console, not just the message
  },
  cloudWatch: {
    // This is provided as an example only. In order to log to CloudWatch Logs, your environment
    // must be configured to use the AWS SDK.
    region: 'us-west-2', // This is optional. The following environment variables will be
                         // used, if set, in the following order:
                         // AWS_CLOUDWATCH_LOGS_REGION; AWS_CLOUDWATCH_REGION; AWS_DEFAULT_REGION
    logGroup: `/${process.env.NODE_ENV}/goodware/log`,
  },
  categories: {
    // These are named loggers aka Winston categories. Each category can specify which transports
    // to use: console, file, or cloudWatch, and at which logging level. All categories inherit
    // from the default category. Logging levels are configurable. By default they are:
    // fail, error, warn, info, more, verbose, db, http, debug, silly
    default: {
      console: 'silly',
      cloudWatch: 'off',
      file: 'off',
      errorFile: 'off',
    },
    briefConsole: {
      console: {
        level: 'info',
        data: false,
      },
    },
  },
};

let loggers;
capcon.captureStdout(() => {
  loggers = new Loggers(options);
});;



options.console.data = false;
const briefConsole = new Loggers(options);
capcon.captureStdout(() => {
  briefConsole.log(['info','warn'],'A brief word');
});;

capcon.captureStdout(() =>
  loggers.logger('briefConsole').info('Another brief word')
);;





capcon.captureStdout(()=>
  loggers.logger('briefConsole').log(['info', 'warn'], 'Danger Will Robinson')
);;

capcon.captureStdout(()=>
  loggers.logger('briefConsole').info(['warn'], 'Danger Will Robinson')
);;

capcon.captureStdout(()=>
  loggers.logger('briefConsole').child('warn').log( {warn: 0, info: true }, 'This is a silly example')
);;

capcon.captureStdout(() => {
  loggers.error('Message', new Error('Inner error'));
});;

capcon.captureStdout(() => {
  loggers.error({message: 'Message', error: new Error('Inner Error')});
});;

capcon.captureStdout(() => {
  loggers.log(new Error('Something went wrong'));
});;

capcon.captureStdout(() => {
  briefConsole.log({error: new Error('Something went wrong')});
});;

const threadLogger = loggers.logger('threads');
capcon.captureStdout(() => {
  threadLogger.info('Started a thread');
});;


capcon.captureStdout(() => {
  loggers.log(['sql', 'thread'], new Error('Inner error'));
});;

capcon.captureStdout(() => {
  loggers.warn({tags: ['sql', 'thread'], error: new Error('Inner error')});
});;


options.categories = {
  default: {
    console: 'db',
    tags: {
      sql: 'off', // Disable for all transports for all categories. Log entries with warn and error
                  // levels are logged.
    },
  },
  one: {
    tags: {
      sql: {
        // Fine-tune filtering for category 'one.' All of these keys are optional.
        allowLevel: 'off', // Enable tag filtering for all log entries regardless of their levels. 'off' is needed
          // because the default is 'warn.'
        level: 'db', // Set the log entry's level to 'more'
        // Log entries are sent to all transports by default (console, file, errorFile, cloudWatch). Each transport
        // can be overridden:
        file: 'verbose', // Send a log entry with the 'sql' tag to the file transport if the log entry's severity
          // level is equal to or greater than 'verbose'
        console: 'on', // Send a log entry with the 'sql' tag to the console transport
        other: 'off', // Do not send a log entry with the 'sql' tag to CloudWatch
      },
    },
  },
  two: {
    tags: {
      sql: 'on', // Send all log entries with the 'sql' tag to all transports for category two
    },
  },
};

loggers2 = new Loggers(options);

capcon.captureStdout(() => {
  loggers2.silly('This is not logged'); // This is not logged because the console's level is set to db
  loggers2.db(['sql'], 'This is not logged'); // This is not logged because the sql tag is disabled by default
  loggers2.db('Logging to general (level = db)'); // This is logged because the sql tag is not specified
  loggers2.logger('one').silly(['sql'], 'Logging SQL to one (level = db)'); // sql is enabled via {level: 'db'}
  loggers2.logger('two').info(['sql'], 'Logging SQL to two (level = info)'); // sql is enabled via 'on'
});;

capcon.captureStdout(() => {
  loggers.error('Operation failed', {error: new Error('An error'), counter: 5});
});;

capcon.captureStdout(() => {
  loggers.info({a: 5}, {a: 10});
});;


capcon.captureStdout(() => {
  const child1 = loggers.child(['tag1'], { requestId: 'AEF1X' });
  child1.info('First child');
  const child2 = child1.child(null, { operationId: 10 });
  child2.warn('Second child');
});;
capcon.captureStdout(() => {
  loggers.logger('dog').child('dogTag').child(null, { userId: 101 }).logger('poodle')
  .debug('Woof!');
});;

capcon.captureStdout(() => {
  const stack = loggers.stack(); // Use the default name
  stack.push(loggers.child('tag1'));
  stack.push(stack.top().child('tag2'));
  stack.top().info('tag1 and tag2 are added');
  stack.pop();
  stack.top().info('Only tag1 is added');
});;

loggers.logger('dog', loggers.child('dogTag'));

capcon.captureStdout(() => {
  loggers.info({category: 'dog', message: 'woof' });
});;



const TaskLogger = require('@goodware/log/TaskLogger');
{
var output = '';
capcon.startCapture(process.stdout, function (stdout) {
  output += stdout;
});
await TaskLogger.execute(loggers, ()=>true);
capcon.stopCapture(process.stdout);
output;
};

{
var output = '';
capcon.startCapture(process.stdout, function (stdout) {
  output += stdout;
});
await TaskLogger.execute(loggers.child('info'), ()=>true);
capcon.stopCapture(process.stdout);
output;
};

{
var output = '';
capcon.startCapture(process.stdout, function (stdout) {
  output += stdout;
});
try {
  await TaskLogger.execute(loggers, ()=>{throw new Error('Failed');});
} catch (error) {
 //
}
capcon.stopCapture(process.stdout);
output;
};



await loggers.flush();;


var output = '';
capcon.startCapture(process.stdout, function (stdout) {
  output += stdout;
});
await loggers.stop();
capcon.stopCapture(process.stdout);
output;;
